part of web_api;
class _ParameterResolver{

  Serializer serializer;
  Utf8Codec utf8;

  Object parseBody(HttpRequest request, ParameterMirror bodyParameterMirror) async{
    String bodyString = await utf8.decodeStream(request);
    Object bodyInstance = serializer.decode(bodyString,bodyParameterMirror.type.reflectedType);
    return bodyInstance;
  }

  Object resolveResourceParameter(HttpRequest request, ParameterMirror parameterMirror, _RequestMapping mapping){
    List<String> mappingUriSegments = mapping.url.split("/").where((s)=>s.length>0).toList();
    List<String> requestUriSegments = request.uri.pathSegments;
    String parameterName = MirrorSystem.getName(parameterMirror.simpleName);
    int segmentIndex = mappingUriSegments.indexOf("{"+parameterName+"}");
    String value = requestUriSegments[segmentIndex];
    Object result;
    if(parameterMirror.type.reflectedType == int){
      result = int.parse(value);
    }
    else{
      result=value;
    }
    return result;
  }

  Object resolveQueryParameter(HttpRequest request, ParameterMirror parameterMirror, _RequestMapping mapping){
    Object result;
    String value = request.uri.queryParameters[MirrorSystem.getName(parameterMirror.simpleName)];
    if(parameterMirror.type.reflectedType == int){
      result=int.parse(value);
    }
    else{
      result = value;
    }
    return result;
  }


  Future<List<Object>> resolve(HttpRequest request, _RequestMapping mapping) async{
    List<Object> paramList = new List<Object>();
    ClassMirror controllerClassMirror = reflectType(mapping.controller);
    MethodMirror controllerMethodMirror  = controllerClassMirror.declarations.values.where((mirror)=>mirror.simpleName == mapping.controllerMethod).first;
    for(ParameterMirror paramMirror in controllerMethodMirror.parameters){
      Object value;
      if(paramMirror.simpleName == mapping.bodyVariable){
        value = await parseBody(request,paramMirror);
        paramList.add(value);
      }
      else if(mapping.queryParams.contains(paramMirror.simpleName)){
        paramList.add(resolveQueryParameter(request,paramMirror,mapping));
      }
      else{
        paramList.add(resolveResourceParameter(request,paramMirror,mapping));
      }
    }
    return paramList;
  }

  _ParameterResolver(Utf8Codec utf8, Serializer serializer){
      this.utf8=utf8;
      this.serializer=serializer;
  }

}
