part of web_api;
class ResponseBuilder{

  Serializer serializer = new Serializer.Json();
  LinkAppander linkAppander = new LinkAppander();

  bool _shouldHandleResponse(_RequestMapping mapping){
    bool result = true;
    ClassMirror controllerClassMirror = reflectClass(mapping.controller);
    MethodMirror apiMethodMirror = controllerClassMirror.declarations[mapping.controllerMethod];
    for(InstanceMirror metadataMirror in apiMethodMirror.metadata){
      if(metadataMirror.reflectee is HandlesResponse){
        result = false;
      }
    }
    return result;
  }

  String _buildResultBody(Object result, _RequestMapping mapping, HttpRequest request, Controller controller){
    Map<String,dynamic> resultMap = serializer.toMap(result);
    linkAppander.appendLinks(resultMap,result,mapping,request,controller);
    String responseBody = serializer.encode(resultMap);
    return responseBody;
  }

  String _buildListResultBody(List<Object> result, _RequestMapping mapping, HttpRequest request,Controller controller){
    List<Map> resultElements = new List<Map>();
    int index = 0;
    for(Object resultElement in result){
      Map<String,dynamic> resultElementMap = serializer.toMap(resultElement);
      linkAppander.appendLinks(resultElementMap,resultElement,mapping,request,controller,index);
      resultElements.add(resultElementMap);
      index++;
    }
    String responseBody = serializer.encode(resultElements);
    return responseBody;
  }

  Future success(HttpRequest request, _RequestMapping mapping, Object result, Controller controller)async{
    if(!_shouldHandleResponse(mapping)){
      return;
    }
    String responseBody='';
    String contentType="text/plain";
    if(result is String){
      responseBody = result;
    }
    else if(result is num){
      responseBody = result.toString();
    }
    else if(result is List){
      contentType="application/json";
      if(result.length>0){
        if(result.first is String || result.first is num){
          responseBody = serializer.encode(result);
        }
        else{
          responseBody = _buildListResultBody(result,mapping,request,controller);
        }
      }
      else{
        responseBody='[]';
      }
    }
    else{
      responseBody = _buildResultBody(result,mapping,request,controller);
      contentType="application/json";
    }
    request.response.statusCode = mapping.successStatus;
    await request.response
      ..headers.set("Content-Type",contentType)
      ..write(responseBody)
      ..close();
  }

  Future controllerException(HttpRequest request, ControllerException e)async{
    request.response.statusCode=e.statusCode;
    await request.response
      ..write(e.message)
      ..close();
  }

  Future exception(HttpRequest request, dynamic e)async{
    request.response.statusCode=HttpStatus.INTERNAL_SERVER_ERROR;
    await request.response
      ..write(e.toString())
      ..close();
  }

  Future unknownRoute(HttpRequest request)async{
    request.response.statusCode=HttpStatus.NOT_FOUND;
    await request.response.close();
  }

  Future allowOption(HttpRequest request)async{
    request.response.statusCode=HttpStatus.OK;
    await request.response.close();
  }

  ResponseBuilder(Serializer serializer, LinkAppander linkAppender){
      this.serializer=serializer;
      this.linkAppander=linkAppender;
  }


}
